#!/usr/bin/python
#
# Copyright (C) 2010 Brad Misik <temugen@bradmisik.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2, as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from xml.etree import ElementTree as etree
from string import Template
import math
import subprocess
import re
import random

class Arc(object):
    def __init__(self, center, radius, start_angle, end_angle, width, color):
        self.center = center
        self.radius = radius
        self.start_angle = start_angle
        self.end_angle = end_angle
        self.width = width
        self.color = color

    def tosvg(self):
        svg = etree.Element("path")

        #coordinates for center of circle
        xc, yc = self.center

        #angle between control points
        dcp = (self.end_angle - self.start_angle) / 3

        #inner and outer radii
        ri = self.radius - (self.width/2)
        ro = self.radius + (self.width/2)

        #coordinates for the inner arc
        x1i = xc + ri * math.cos(self.start_angle)
        y1i = yc - ri * math.sin(self.start_angle)
        x2i = xc + ri * math.cos(self.end_angle)
        y2i = yc - ri * math.sin(self.end_angle)
        cpx1i = xc + ri * math.cos(self.start_angle + dcp)
        cpy1i = yc - ri * math.sin(self.start_angle + dcp)
        cpx2i = xc + ri * math.cos(self.start_angle + 2*dcp)
        cpy2i = yc - ri * math.sin(self.start_angle + 2*dcp)

        #coordinates for the outer arc
        x1o = xc + ro * math.cos(self.start_angle)
        y1o = yc - ro * math.sin(self.start_angle)
        x2o = xc + ro * math.cos(self.end_angle)
        y2o = yc - ro * math.sin(self.end_angle)
        cpx1o = xc + ro * math.cos(self.start_angle + dcp)
        cpy1o = yc - ro * math.sin(self.start_angle + dcp)
        cpx2o = xc + ro * math.cos(self.start_angle + 2*dcp)
        cpy2o = yc - ro * math.sin(self.start_angle + 2*dcp)

        #path data
        d = 'M%d,%d C%d,%d %d,%d %d,%d L%d,%d C%d,%d %d,%d %d,%d L%d,%d' % \
                (x1i,y1i, cpx1i,cpy1i, cpx2i,cpy2i, x2i,y2i, x2o,y2o, \
                 cpx2o,cpy2o, cpx1o,cpy1o, x1o,y1o, x1i,y1i)
        svg.attrib['d'] = '%s' % d
        svg.attrib['fill'] = self.color
        svg.attrib['stroke'] = 'black'
        svg.attrib['stroke-width'] = '1'

        return etree.tostring(svg)

def random_color():
    r = random.randint(0, 255)
    g = random.randint(0, 255)
    b = random.randint(0, 255)
    return 'rgb(%d,%d,%d)' % (r, g, b)

def parse_ureadahead_blocks():
    block_size = 4096 * 100
    #stored as {macroblock_num: {'filename1': size}}
    blocks = {}

    ureadahead_cmd = 'sudo ureadahead --dump'
    ureadahead = subprocess.Popen(ureadahead_cmd.split(), stdout=subprocess.PIPE)
    dump = ureadahead.communicate()[0]

    lines = dump.splitlines()
    for line in lines:
        filenames = re.findall('(/.*?)\s', line)
        if len(filenames) >= 1:
            filename = filenames[0]

        bytes = re.findall('\ (.*?)\ bytes', line)
        if len(bytes) >= 1:
            size = int(bytes[0])
            location = int(re.findall('\(at\ (.*?)\)', line)[0])
            block_key = int(math.floor(location / block_size))
            if block_key not in blocks:
                blocks[block_key] = {}
            if filename in blocks[block_key]:
                blocks[block_key][filename] += size
            else:
                blocks[block_key][filename] = size

    files = {}
    simplified_blocks = {}
    for block in blocks:
        (filename, size) = sorted(blocks[block].iteritems(), key=lambda (k,v): (v,k))[-1]
        if filename not in files:
            files[filename] = random_color()
        simplified_blocks[block] = (filename, files[filename])
    return simplified_blocks

if __name__ == '__main__':
    blocks = parse_ureadahead_blocks()

    #svg template xml
    f = open('template.svg')
    template = Template(f.read())
    f.close()

    #tunable drawing properties
    block_width = 5
    block_length = 7
    image_diameter = 1000
    unused_block_color = 'white'
    radius = 10

    #draw the blocks
    gfx = []
    image_center = (image_diameter / 2, image_diameter / 2)
    angle = 0
    block_start = min(blocks)
    block_end = max(blocks) + 1
    while block_start < block_end:
        ring_count = int(math.floor((2 * math.pi * radius) / block_length))
        ring_end = block_start + ring_count
        sweep_angle = (2 * math.pi) / ring_count
        for block in range(block_start, min(ring_end, block_end)):
            if block in blocks:
                (filename, color) = blocks[block]
                arc = Arc(image_center, radius, angle, angle + sweep_angle,
                        block_width, color)
            else:
                arc = Arc(image_center, radius, angle, angle + sweep_angle,
                        block_width, unused_block_color)
            gfx.append(arc.tosvg())
            angle += sweep_angle
        block_start = ring_end
        radius += block_width

    #fill in the template and send our svg image to stdout
    print template.substitute(data="\n".join(gfx),
            diameter=image_diameter)
